
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.011.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "role for Single Sign On (SSO) authentication",
  "dist_author" => [
    "Nicolas Franck, C<< <nicolas.franck at ugent.be> >>"
  ],
  "dist_name" => "Plack-Auth-SSO",
  "dist_version" => "0.0136",
  "license" => "perl",
  "module_name" => "Plack::Auth::SSO",
  "recursive_test_files" => 1,
  "requires" => {
    "Authen::CAS::Client" => 0,
    "Clone" => 0,
    "Data::UUID" => 0,
    "Data::Util" => 0,
    "LWP::Protocol::https" => 0,
    "LWP::UserAgent" => 0,
    "Moo" => 0,
    "Moo::Role" => 0,
    "Plack" => 0,
    "Plack::Middleware::Session" => 0,
    "URI" => 0,
    "URI::QueryParam" => 0,
    "WWW::ORCID" => "0.0401",
    "perl" => "v5.10.1"
  },
  "test_requires" => {
    "Dancer::Middleware::Rebase" => 0,
    "Devel::Cover" => 0,
    "HTTP::Cookies" => 0,
    "HTTP::Request::Common" => 0,
    "LWP::UserAgent" => 0,
    "Plack::Builder" => 0,
    "Plack::Session" => 0,
    "Plack::Test" => 0,
    "Plack::Test::Server" => 0,
    "Test::Exception" => 0,
    "Test::More" => 0,
    "URI::Escape" => 0
  }
);


my %fallback_build_requires = (
  "Dancer::Middleware::Rebase" => 0,
  "Devel::Cover" => 0,
  "HTTP::Cookies" => 0,
  "HTTP::Request::Common" => 0,
  "LWP::UserAgent" => 0,
  "Module::Build" => "0.28",
  "Plack::Builder" => 0,
  "Plack::Session" => 0,
  "Plack::Test" => 0,
  "Plack::Test::Server" => 0,
  "Test::Exception" => 0,
  "Test::More" => 0,
  "URI::Escape" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
